@@ -1,12 +1,14 @@
-context("Kaplan-meier plot")
+# context("Kaplan-meier plot")
 
 library(survival)
+library(ggplot2)
 # data(colon);data(pbc)
 
 test_that("Run jskm", {
   fit <- survfit(Surv(time, status) ~ rx, data = colon)
   expect_is(jskm(fit, timeby = 500, table = T, pval = T), "gg")
   expect_is(jskm(fit,
+    med = T,
     table = T, pval = T, label.nrisk = "No. at risk", timeby = 365, xlabs = "Time(Day)", ylabs = "Survival", marks = F, xlims = c(0, 3500), ylims = c(0.25, 1),
     ystratalabs = c("Obs", "Lev", "Lev + 5FU"), ystrataname = "rx"
   ), "gg")
@@ -20,9 +22,9 @@ test_that("Run jskm", {
   expect_is(jskm(fit, cluster.option = "cluster", cluster.var = "id", pval = T), "gg")
   expect_warning(jskm(fit, cluster.option = "frailty", cluster.var = "id", pval = T))
 
-  fit3 <- survfit(Surv(time, status) ~ rx + frailty(id), data = colon)
-  expect_is(jskm(fit3), "gg")
-  expect_is(jskm(fit3, pval = T), "gg")
+  # fit3 <- survfit(Surv(time, status) ~ rx + frailty(id), data = colon)
+  # expect_is(jskm(fit3), "gg")
+  # expect_is(jskm(fit3, pval = T), "gg")
 })
 
 
@@ -36,7 +38,7 @@ test_that("Run svyjskm", {
   s1 <- survey::svykm(Surv(time, status > 0) ~ sex, design = dpbc, se = T)
   expect_is(svyjskm(s1, ci = T), "gg")
   expect_is(svyjskm(s1, ci = F, ylabs = "Suvrival (%)", surv.scale = "percent"), "gg")
-  expect_is(svyjskm(s1, table = T, pval = T, design = dpbc), "gg")
+  expect_is(svyjskm(s1, table = T, pval = T, design = dpbc, med = T), "gg")
   expect_is(svyjskm(s1, ci = T, cumhaz = T), "gg")
   expect_is(svyjskm(s1, ci = F, cumhaz = T), "gg")
   s2 <- survey::svykm(Surv(time, status > 0) ~ sex, design = dpbc, se = F)
